@@ -32,7 +32,7 @@
 #ifndef lint
 static char copyright[] =
 "@(#) Copyright 1994 Purdue Research Foundation.\nAll rights reserved.\n";
-static char *rcsid = "$Id: arg.c,v 1.45 2005/08/08 19:42:56 abe Exp $";
+static char *rcsid = "$Id: arg.c,v 1.46 2006/03/27 23:04:25 abe Exp $";
 #endif
 
 
@@ -128,7 +128,6 @@ ck_file_arg(i, ac, av, fv, rs, sbp)
 	struct stat *sbp;	/* if non-NULL, pointer to stat(2) buffer
 				 * when argument count == 1 */
 {
-	unsigned char ad, an;
 	short err = 0;
 	char *fnm, *fsnm, *path;
 	int fsm, ftype, j;
@@ -146,6 +145,7 @@ ck_file_arg(i, ac, av, fv, rs, sbp)
 #endif	/* defined(CKFA_EXPDEV) */
 
 #if	defined(HASPROCFS)
+	unsigned char ad, an;
 	int pfsnl = -1;
 	pid_t pid;
 	struct procfsid *pfi;
@@ -332,7 +332,11 @@ ck_file_arg(i, ac, av, fv, rs, sbp)
 	     */
 		if (!fnm || fnm == path) {
 		    sfp->name = fnm;
+
+#if	defined(HASPROCFS)
 		    an = 0;
+#endif	/* defined(HASPROCFS) */
+
 		} else {
 		    if (!(sfp->name = mkstrcpy(fnm, (MALLOC_S *)NULL))) {
 			(void) fprintf(stderr,
@@ -340,11 +344,19 @@ ck_file_arg(i, ac, av, fv, rs, sbp)
 			safestrprt(fnm, stderr, 1);
 			Exit(1);
 		    }
+
+#if	defined(HASPROCFS)
 		    an = 1;
+#endif	/* defined(HASPROCFS) */
+
 		}
 		if (!fsnm || fsnm == path) {
 		    sfp->devnm = fsnm;
+
+#if	defined(HASPROCFS)
 		    ad = 0;
+#endif	/* defined(HASPROCFS) */
+
 		} else {
 		    if (!(sfp->devnm = mkstrcpy(fsnm, (MALLOC_S *)NULL))) {
 			(void) fprintf(stderr,
@@ -352,7 +364,11 @@ ck_file_arg(i, ac, av, fv, rs, sbp)
 			safestrprt(fsnm, stderr, 1);
 			Exit(1);
 		    }
+
+#if	defined(HASPROCFS)
 		    ad = 1;
+#endif	/* defined(HASPROCFS) */
+
 		}
 		if (!(sfp->aname = mkstrcpy(av[i], (MALLOC_S *)NULL))) {
 		    (void) fprintf(stderr,
@@ -937,7 +953,7 @@ enter_dir(d, descend)
 	Dstkn = Dstkx = 0;
 	Dstk = (char **)NULL;
 	(void) stkdir(dn);
-	av[0] = (dn == d) ? mkstrcpy(dn, (MALLOC_S)NULL) : dn;
+	av[0] = (dn == d) ? mkstrcpy(dn, (MALLOC_S *)NULL) : dn;
 	av[1] = (char *)NULL;
 	dn = (char *)NULL;
 	if (!ck_file_arg(0, 1, av, 1, 1, &sb)) {
@@ -2002,7 +2018,7 @@ enter_uid(us)
 		}
 	    }
 	    if (nn) {
-		if (!(lp = mkstrcpy(lnm, (MALLOC_S)NULL))) {
+		if (!(lp = mkstrcpy(lnm, (MALLOC_S *)NULL))) {
 		    (void) fprintf(stderr, "%s: no space for login: ", Pn);
 		    safestrprt(lnm, stderr, 1);
 		    Exit(1);
