@@ -31,7 +31,7 @@
 
 
 /*
- * $Id: dlsof.h,v 1.38 2005/08/08 19:49:23 abe Exp $
+ * $Id: dlsof.h,v 1.39 2006/03/27 23:29:03 abe Exp $
  */
 
 
@@ -149,7 +149,11 @@ struct vop_getextattr_args;
 struct vop_deleteextattr_args;
 struct vop_setextattr_args;
 #include <ufs/ufs/extattr.h>
+#define	psignal	LSOF_psignal
+#define	panicstr bp
 #include <ufs/ufs/ufsmount.h>
+#undef	psignal
+#undef	panicstr
 #undef	_KERNEL
 # endif	/* defined(HAS_UFS1_2) */
 
@@ -297,6 +301,15 @@ struct vop_advlock_args { int dummy; };	/* to pacify lf_advlock() prototype */
 #define	MALLOC_DECLARE(type)	extern struct malloc_type type[1]
 					/* to pacify <sys/lockf.h> */
 #define	_KERNEL
+
+#   if	defined(HAS_SYS_SX_H)
+#include <sys/sx.h>
+#   endif	/* defined(HAS_SYS_SX_H) */
+
+#   if	defined(HAS_SI_PRIV)
+#include <fs/devfs/devfs_int.h>
+#   endif	/* defined(SI_PRIV) */
+
 #include <fs/devfs/devfs.h>
 #undef	_KERNEL
 #  endif	/* FREEBSDV>=5000 */
@@ -362,6 +375,7 @@ typedef	u_long		KA_T;
 #define MALLOC_P	void
 #define FREE_P		MALLOC_P
 #define MALLOC_S	size_t
+#define	MAXSYSCMDL	MAXCOMLEN	/* max system command name length */
 
 # if	defined(N_UNIXV)
 #define	N_UNIX_TMP(x)	#x
