@@ -35,7 +35,7 @@
 #ifndef lint
 static char copyright[] =
 "@(#) Copyright 1994 Purdue Research Foundation.\nAll rights reserved.\n";
-static char *rcsid = "$Id: dnode1.c,v 1.10 2005/05/11 12:56:47 abe Exp $";
+static char *rcsid = "$Id: dnode1.c,v 1.11 2005/08/08 19:50:23 abe Exp $";
 #endif
 
 
@@ -132,7 +132,7 @@ read_vxnode(v, vfs, dev, devs, rdev, rdevs)
 /*
  * Record inode number.
  */
-	Lf->inode = (unsigned long)i.i_number;
+	Lf->inode = (INODETYPE)i.i_number;
 	Lf->inp_ty = 1;
 /*
  * Record size.
