@@ -136,7 +136,7 @@ _PROTOTYPE(static struct pst_vm_status *read_vmreg,(struct pst_status *p,
 void
 gather_proc_info()
 {
-	int cwds, fd, *fds, fdsa, i, j, k, l, nf, np, rtds;
+	int cwds, fd, *fds, fdsa, i, j, l, nf, np, rtds;
 	struct pst_fileinfo2 *f;
 	long flag;
 	KA_T ka, na;
@@ -299,7 +299,7 @@ gather_proc_info()
 		if (Fsv & FSV_FA) {
 		    ka = (((KA_T)(f->psf_hi_fileid & 0xffffffff) << 32)
 		       |  (KA_T)(f->psf_lo_fileid & 0xffffffff));
-		    if (Lf->fsa = ka)
+		    if ((Lf->fsa = ka))
 			Lf->fsv |= FSV_FA;
 		}
 		if (Fsv & FSV_FG) {
