@@ -33,12 +33,12 @@
 
 
 /*
- * $Id: dproto.h,v 1.3 2004/10/17 21:58:08 abe Exp $
+ * $Id: dproto.h,v 1.4 2005/05/11 12:57:04 abe Exp $
  */
 
 
-_PROTOTYPE(extern int is_file_named,(char *p, int cd));
 _PROTOTYPE(extern int get_max_fd,(void));
+_PROTOTYPE(extern int is_file_named,(char *p, int cd));
 _PROTOTYPE(extern void process_finfo,(struct pst_filedetails *pd, struct pst_fid *opfid, struct psfileid *psfid, KA_T na));
 _PROTOTYPE(extern void process_socket,(struct pst_fileinfo2 *f,
 				       struct pst_socket *s));
@@ -48,6 +48,10 @@ _PROTOTYPE(extern KA_T read_det,(struct pst_fid *ki, uint32_t hf, uint32_t lf,
 				 struct pst_filedetails *pd));
 _PROTOTYPE(extern struct pst_socket *read_sock,(struct pst_fileinfo2 *f));
 
+#if	defined(HASIPv6)
+_PROTOTYPE(extern struct hostent *gethostbyname2,(char *nm, int proto));
+#endif	/* defined(HASIPv6) */
+
 #if	defined(HASVXFS)
 _PROTOTYPE(extern int read_vxnode,(struct vnode *v, struct l_vfs *vfs, dev_t *dev));
 #endif	/* defined(HASVXFS) */
