@@ -47,7 +47,7 @@ _PROTOTYPE(static void printpsproto,(uint32_t p));
 
 
 /*
- * Local macroes
+ * Local macros
  */
 
 #if	defined(HASIPv6)
@@ -160,12 +160,12 @@ print_tcptpi(nl)
 {
 	char *cp = (char *)NULL;
 	char  sbuf[128];
-	int i, t;
+	int i;
 	int ps = 0;
 	unsigned int u;
 
 	if (Ftcptpi & TCPTPI_STATE) {
-	    switch ((t = Lf->lts.type)) {
+	    switch (Lf->lts.type) {
 	    case 0:				/* TCP */
 		switch ((i = Lf->lts.state.i)) {
 		case PS_TCPS_CLOSED:
@@ -1043,21 +1043,21 @@ process_stream(f)
  * pst_fileino2 structure.  Enter size from stream head's structure,
  * if requested.
  */
-	if (f->psf_hi_fileid != s[0].val.head.pst_hi_fileid
-	 |  f->psf_lo_fileid != s[0].val.head.pst_lo_fileid
-	 |  f->psf_hi_nodeid != s[0].val.head.pst_hi_nodeid
-	 |  f->psf_lo_nodeid != s[0].val.head.pst_lo_nodeid) {
+	if ((f->psf_hi_fileid != s[0].val.head.pst_hi_fileid)
+	 |  (f->psf_lo_fileid != s[0].val.head.pst_lo_fileid)
+	 |  (f->psf_hi_nodeid != s[0].val.head.pst_hi_nodeid)
+	 |  (f->psf_lo_nodeid != s[0].val.head.pst_lo_nodeid)) {
 	    enter_nm("no matching stream data available");
 	    return;
 	}
 	if (Fsize) {
-	    if (Lf->access = 'r') {
+	    if (Lf->access == 'r') {
 		Lf->sz = (SZOFFTYPE)s[0].val.head.pst_rbytes;
 		Lf->sz_def = 1;
-	    } else if (Lf->access = 'w') {
+	    } else if (Lf->access == 'w') {
 		Lf->sz = (SZOFFTYPE)s[0].val.head.pst_wbytes;
 		Lf->sz_def = 1;
-	    } else if (Lf->access = 'u') {
+	    } else if (Lf->access == 'u') {
 		Lf->sz = (SZOFFTYPE)s[0].val.head.pst_rbytes
 		       + (SZOFFTYPE)s[0].val.head.pst_wbytes;
 		Lf->sz_def = 1;
