@@ -31,7 +31,7 @@
 
 
 /*
- * $Id: machine.h,v 1.19 2005/08/29 10:21:48 abe Exp $
+ * $Id: machine.h,v 1.20 2006/03/28 20:45:24 abe Exp $
  */
 
 
@@ -49,6 +49,19 @@
 #  endif	/* !defined( __STDC_EXT__) */
 
 #include <sys/_inttypes.h>
+
+#  if	HPUXV>=1123
+#undef	LSOF_XOPEN_SOURCE_EXTENDED
+#   if	defined(_XOPEN_SOURCE_EXTENDED)
+#define	LSOF_XOPEN_SOURCE_EXTENDED _XOPEN_SOURCE_EXTENDED
+#undef	_XOPEN_SOURCE_EXTENDED
+#   endif	/* defined(_XOPEN_SOURCE_EXTENDED) */
+#include <netdb.h>
+#   if	defined(LSOF_XOPEN_SOURCE_EXTENDED)
+#define	_XOPEN_SOURCE_EXTENDED LSOF_XOPEN_SOURCE_EXTENDED
+#undef	LSOF_XOPEN_SOURCE_EXTENDED
+#   endif	/* defined(_XOPEN_SOURCE_EXTENDED) */
+#  endif	/* HPUXV>=1123 */
 # endif	/* defined(__GNUC__) */
 
 
@@ -163,10 +176,19 @@
  * FSV_DEFAULT defines the default set of file structure values to list.
  * It defaults to zero (0), but may be made up of a combination of the
  * FSV_* symbols from lsof.h.
+ *
+ *   HASNOFSADDR  -- has no file structure address
+ *   HASNOFSFLAGS -- has no file structure flags
+ *   HASNOFSCOUNT -- has no file structure count
+ *   HASNOFSNADDR -- has no file structure node address
  */
 
 #define	HASFSTRUCT	1
 /* #define	FSV_DEFAULT	FSV_? | FSV_? | FSV_? */
+/* #define	HASNOFSADDR	1	has no file structure address */
+/* #define	HASNOFSFLAGS	1	has no file structure flags */
+/* #define	HASNOFSCOUNT	1	has no file structure count */
+/* #define	HASNOFSNADDR	1	has no file structure node address */
 
 
 /*
@@ -437,7 +459,7 @@
 
 #define	HASSETLOCALE	1
 #define	HASWIDECHAR	1
-/* #define	WIDECHARINCL	<wchar.h>	*/
+#define	WIDECHARINCL	<wctype.h>
 
 
 /*
