@@ -31,7 +31,7 @@
 
 
 /*
- * $Id: dlsof.h,v 1.15 2005/08/29 10:23:33 abe Exp $
+ * $Id: dlsof.h,v 1.16 2006/03/27 22:57:11 abe Exp $
  */
 
 
@@ -80,6 +80,17 @@ typedef	unsigned long	KA_T;
 #define MALLOC_P	void
 #define FREE_P		MALLOC_P
 #define MALLOC_S	size_t
+#define	MAXSYSCMDL	15	/* max system command name length
+				 *   This value should be obtained from a
+				 * header file #define, but no consistent one
+				 * exists.  Some versions of the Linux kernel
+				 * have a hard-coded "char comm[16]" command
+				 * name member of the task structured
+				 * definition in <linux/sched.h>, while others
+				 * have a "char comm[TASK_COMM_LEN]" member
+				 * with TASK_COMM_LEN #define'd to be 16.
+				 *   Hence, a universal, local definition of
+				 * 16 is #define'd here. */
 #define	PROCFS		"/proc"
 #define QSORT_P		void
 #define	READLEN_T	size_t
