@@ -32,7 +32,7 @@
 #ifndef	lint
 static char copyright[] =
 "@(#) Copyright 1997 Purdue Research Foundation.\nAll rights reserved.\n";
-static char *rcsid = "$Id: dmnt.c,v 1.13 2004/07/06 19:19:44 abe Exp $";
+static char *rcsid = "$Id: dmnt.c,v 1.16 2005/08/29 10:23:33 abe Exp $";
 #endif
 
 
@@ -421,6 +421,7 @@ readmnt()
 	char *ln;
 	struct mounts *mp;
 	FILE *ms;
+	int nfs;
 	struct stat sb;
 
 	if (Lmi || Lmist)
@@ -437,7 +438,7 @@ readmnt()
  * Read mount table entries.
  */
 	while (fgets(buf, sizeof(buf), ms)) {
-	    if (get_fields(buf, (char *)NULL, &fp) < 3
+	    if (get_fields(buf, (char *)NULL, &fp, (int *)NULL, 0) < 3
 	    ||  !fp[0] || !fp[1] || !fp[2])
 		continue;
 	/*
@@ -485,6 +486,11 @@ readmnt()
 	    }
 	    if (*dn != '/')
 		continue;
+	/*
+	 * Detect an NFS mount point.
+	 */
+	    if (!(nfs = strcasecmp(fp[2], "nfs")) && !HasNFS)
+		HasNFS = 1;
 	/*
 	 * Stat() the directory.
 	 */
@@ -528,11 +534,12 @@ readmnt()
 	    mp->next = Lmi;
 	    mp->dev = ((mp->ds = ds) & SB_DEV) ? sb.st_dev : 0;
 	    mp->rdev = (ds & SB_RDEV) ? sb.st_rdev : 0;
-	    mp->inode = (ds & SB_INO) ? sb.st_ino : 0;
+	    mp->inode = (INODETYPE)((ds & SB_INO) ? sb.st_ino : 0);
 	    mp->mode = (ds & SB_MODE) ? sb.st_mode : 0;
-	    if (strcasecmp(fp[2], "nfs") == 0) {
-		HasNFS = 1;
+	    if (!nfs) {
 		mp->ty = N_NFS;
+		if (HasNFS < 2)
+		    HasNFS = 2;
 	    } else
 		mp->ty = N_REGLR;
 
