@@ -32,10 +32,17 @@
 #ifndef lint
 static char copyright[] =
 "@(#) Copyright 1997 Purdue Research Foundation.\nAll rights reserved.\n";
-static char *rcsid = "$Id: dstore.c,v 1.1 97/12/30 08:15:40 abe Exp $";
+static char *rcsid = "$Id: dstore.c,v 1.2 2005/05/11 12:53:27 abe Exp $";
 #endif
 
 
 #include "lsof.h"
 
-int HasNFS = 0;				/* 1 == there is an NFS file system */
+int HasNFS = 0;				/* NFS mount point status:
+					 *     1 == there is an NFS mount point,
+					 *          but its device number is
+					 *          unknown
+					 *     2 == there is an NFS mount point
+					 *          and its device number is
+					 *          known
+					 */
