@@ -32,7 +32,7 @@
 #ifndef lint
 static char copyright[] =
 "@(#) Copyright 1994 Purdue Research Foundation.\nAll rights reserved.\n";
-static char *rcsid = "$Id: dfile.c,v 1.19 2005/08/29 10:24:25 abe Exp $";
+static char *rcsid = "$Id: dfile.c,v 1.20 2006/03/27 22:57:33 abe Exp $";
 #endif
 
 
@@ -434,6 +434,8 @@ print_v_path(lf)
 	struct lfile *lf;		/* local file structure */
 {
 	char buf[MAXPATHLEN+1];
+	unsigned char del = 0;
+	unsigned char aperr = 0;
 
 # if	defined(HASMNTSTAT)
 	struct stat sb;
@@ -471,18 +473,34 @@ print_v_path(lf)
 		} else {
 
 		/*
-		 * The stat(2) failed.  Ignore the v_path if the error reply
-		 * is ENOENT.  The file has probably been unlinked.  If the
-		 * error reply is EACCES or EPERM, report the v_path, because
-		 * lsof probably lacks permission to apply stat(2) to v_path.
+		 * The stat(2) failed.
+		 *
+		 * If the error reply is ENOENT and the -X option hasn't been
+		 * specified, ignore the v_path.
+		 *
+		 * If the error reply is ENOENT, the -X option has been
+		 * specified and the file's link count is zero, report the
+		 * v_path with the "(deleted)" notation.
+		 *
+		 * If the error reply is EACCES or EPERM, report the v_path,
+		 * followed by "(?)", because lsof probably lacks permission
+		 * to apply stat(2) to v_path.
 		 */
 		    switch (errno) {
 		    case EACCES:
+		    case EPERM:
+			aperr = 1;
 			break;
 		    case ENOENT:
+
+# if	defined(HASXOPT)
+			if (Fxopt && lf->nlink_def && !lf->nlink) {
+			    del = 1;
+			    break;
+			}
+# endif	/* defined(HASXOPT) */
+
 			return(0);
-		    case EPERM:
-			break;
 		    default:
 			return(0);
 		    }
@@ -494,6 +512,10 @@ print_v_path(lf)
 	 * Print the v_path.
 	 */
 	    safestrprt(buf, stdout, 0);
+	    if (del)
+		safestrprt(" (deleted)", stdout, 0);
+	    else if (aperr)
+		safestrprt(" (?)", stdout, 0);
 	    return(1);
 	}
 	return(0);
