@@ -31,7 +31,7 @@
 
 
 /*
- * $Id: dlsof.h,v 1.42 2005/08/29 10:24:25 abe Exp $
+ * $Id: dlsof.h,v 1.44 2007/04/24 16:23:15 abe Exp $
  */
 
 
@@ -95,6 +95,7 @@
 # endif	/* solaris<20600 */
 
 # if	solaris>=80000
+#include <netinet/igmp.h>
 #include <netinet/ip6.h>
 # endif	/* solaris>=80000 */
 
@@ -313,6 +314,7 @@ extern int nlist();
 #define	COMP_P		const void
 #define	CWDLEN		(MAXPATHLEN+1)
 #define DEVINCR		1024		/* device table malloc() increment */
+#define	DINAMEL		32
 #define	DIRTYPE		dirent
 
 # if	solaris>=70000
@@ -321,6 +323,10 @@ typedef	uintptr_t	KA_T;
 typedef	void *		KA_T;
 # endif	/* solaris>=70000 */
 
+# if	solaris>=70000
+#define	KA_T_FMT_X	"0x%p"
+# endif	/* solaris>=70000 */
+
 # if	solaris>=20501
 #define	KMEM		"/dev/mem"
 # else	/* solaris<20501 */
@@ -336,12 +342,7 @@ typedef	void *		KA_T;
 # endif	/* !defined(MAXEND) */
 
 #define MAXSEGS		100		/* maximum text segments */
-#define	DINAMEL		32
-
-# if	solaris>=70000
-#define	KA_T_FMT_X	"0x%p"
-# endif	/* solaris>=70000 */
-
+#define	MAXSYSCMDL	MAXCOMLEN	/* max system command name length */
 #define NETCLNML	8
 #define	N_UNIX		"/dev/ksyms"
 #define	PROCMIN		5		/* processes that make a "good" scan */
@@ -515,6 +516,18 @@ extern int Unof;			/* u_nofiles value */
 #define	VXVOP_NUM	5		/* number of Vvops[] entries */
 
 
+/*
+ * ZFS definitions
+ */
+
+#define	ZVOP_D		0		/* Zvops[] zfs_dvnodeops index */
+#define	ZVOP_E		1		/* Zvops[] zfs_evnodeops index */
+#define	ZVOP_F		2		/* Zvops[] zfs_fvnodeops index */
+#define	ZVOP_SYM	3		/* Zvops[] zfs_symvnodeops index */
+#define	ZVOP_XD		4		/* Zvops[] zfs_xdvnodeops index */
+#define	ZVOP_NUM	5		/* number of Zvops[] entries */
+
+
 /*
  * Kernel name list definitions
  */
@@ -567,4 +580,8 @@ extern int Unof;			/* u_nofiles value */
 #  endif	/* solaris>=80000 */
 # endif	/* defined(HASNCACHE) */
 
+# if	defined(HAS_ZFS) && HAS_ZFS==1
+#define	DIALECT_WARNING "privately defined ZFS structures may be incorrect."
+# endif	/* defined(HAS_ZFS) && HAS_ZFS==1 */
+
 #endif	/* SOLARIS_LSOF_H */
