@@ -32,7 +32,7 @@
 #ifndef lint
 static char copyright[] =
 "@(#) Copyright 1994 Purdue Research Foundation.\nAll rights reserved.\n";
-static char *rcsid = "$Id: dnode.c,v 1.51 2005/08/29 10:24:25 abe Exp $";
+static char *rcsid = "$Id: dnode.c,v 1.53 2007/04/24 16:23:15 abe Exp $";
 #endif
 
 
@@ -41,6 +41,36 @@ static char *rcsid = "$Id: dnode.c,v 1.51 2005/08/29 10:24:25 abe Exp $";
 #undef	fs_bsize
 #include <sys/fs/ufs_inode.h>
 
+#if	defined(HAS_ZFS)
+# if	HAS_ZFS==1
+
+typedef struct zfsvfs {
+	vfs_t	*z_vfs;			/* pointer to VFS */
+} zfsvfs_t;
+
+typedef struct znode_phys {
+	uint64_t	d1[10];
+	uint64_t	zp_size;	/* file size */
+	uint64_t	d2;
+	uint64_t	zp_links;	/* links */
+} znode_phys_t;
+
+typedef struct znode {
+	zfsvfs_t	*z_zfsvfs;	/* pointer to associated vfs */
+	vnode_t		*z_vnode;	/* pointer to associated vnode */
+	uint8_t		d1[16];
+	uint64_t	z_id;		/* node ID */
+	uint8_t		d2[136];
+	znode_phys_t	*z_phys;	/* pointer to persistent znode */
+} znode_t;
+# else	/* HAS_ZFS!=1 */
+#define _KERNEL
+#include <sys/zfs_znode.h>
+#undef	_KERNEL
+# endif	/* HAS_ZFS==1 */
+_PROTOTYPE(static int read_nzn,(KA_T na, KA_T nza, znode_t *z));
+#endif	/* defined(HAS_ZFS) */
+
 
 _PROTOTYPE(static struct l_dev *finddev,(dev_t *dev, dev_t *rdev, int flags));
 
@@ -154,6 +184,8 @@ static KA_T Portvops;		/* port_vnodeops address */
 static KA_T Prvops;		/* prvnodeops address */
 static KA_T Sam1vops;		/* samfs_vnodeops address */
 static KA_T Sam2vops;		/* samfs_client_vnodeops address */
+static KA_T Sam3vops;		/* samfs_vnodeopsp address */
+static KA_T Sam4vops;		/* samfs_client_vnodeopsp address */
 static KA_T Sckvops;		/* sock_vnodeops address */
 static KA_T Sncavops;		/* socknca_vnodeops address */
 static KA_T Stpivops;		/* socktpi_vnodeops address */
@@ -166,6 +198,13 @@ static KA_T Vvops[VXVOP_NUM];	/* addresses of:
 				 *   fdd_chain_vnops (VXVOP_FDDCH),
 				 *   vx_vnodeops (VXVOP_REG)
 				 *   vx_vnodeops_p (VXVOP_REG_P) */
+static KA_T Zvops[ZVOP_NUM];	/* addresses of:
+				 *   zfs_dvnodeops (ZVOPS_D)
+				 *   zfs_evnodeops (ZVOPS_E)
+				 *   zfs_fvnodeops (ZVOPS_F)
+				 *   zfs_symvnodeops (ZVOPS_SYM)
+				 *   zfs_xdvnodeops (ZVOPS_XD)
+				 */
 
 /*
  * Local macros
@@ -663,6 +702,16 @@ process_node(va)
 	struct l_ino vx;
 #endif	/* defined(HASVXFS) */
 
+#if	defined(HAS_ZFS)
+	vfs_t zgvfs;
+	unsigned char zns = 0;
+	unsigned char zps = 0;
+	znode_t zn;
+	znode_phys_t znp;
+	zfsvfs_t zvfs;
+#endif	/* defined(HAS_ZFS) */
+
+
 /*
  * Do first-time only operations.
  */
@@ -674,8 +723,11 @@ process_node(va)
 	if (ft) {
 	    GETVOPS("auvops", Drive_Nl, Auvops);
 	    GETVOPS("avops", Drive_Nl, Avops);
-	    if (!Avops)
+	    if (!Avops) {
 		GETVOPS("Avops", Drive_Nl, Avops);
+		if (!Avops)
+		    GETVOPS("afsops", Drive_Nl, Avops);
+	    }
 	    GETVOPS("ctfsadir", Drive_Nl, Ctfsadir);
 	    GETVOPS("ctfsbund", Drive_Nl, Ctfsbund);
 	    GETVOPS("ctfscdir", Drive_Nl, Ctfscdir);
@@ -706,6 +758,8 @@ process_node(va)
 	    GETVOPS("prvops", Drive_Nl, Prvops);
 	    GETVOPS("sam1vops", Drive_Nl, Sam1vops);
 	    GETVOPS("sam2vops", Drive_Nl, Sam2vops);
+	    GETVOPS("sam3vops", Drive_Nl, Sam3vops);
+	    GETVOPS("sam4vops", Drive_Nl, Sam4vops);
 	    GETVOPS("sckvops", Drive_Nl, Sckvops);
 	    GETVOPS("sncavops", Drive_Nl, Sncavops);
 	    GETVOPS("stpivops", Drive_Nl, Stpivops);
@@ -722,6 +776,16 @@ process_node(va)
 		GETVOPS("vvops", Drive_Nl, Vvops[VXVOP_REG]);
 	    if (VXVOP_REG_P < VXVOP_NUM)
 		GETVOPS("vvops_p", Drive_Nl, Vvops[VXVOP_REG_P]);
+	    if (ZVOP_D < ZVOP_NUM)
+		GETVOPS("zfsdops", Drive_Nl, Zvops[ZVOP_D]);
+	    if (ZVOP_E < ZVOP_NUM)
+		GETVOPS("zfseops", Drive_Nl, Zvops[ZVOP_E]);
+	    if (ZVOP_F < ZVOP_NUM)
+		GETVOPS("zfsfops", Drive_Nl, Zvops[ZVOP_F]);
+	    if (ZVOP_SYM < ZVOP_NUM)
+		GETVOPS("zfssymops", Drive_Nl, Zvops[ZVOP_SYM]);
+	    if (ZVOP_XD < ZVOP_NUM)
+		GETVOPS("zfsxdops", Drive_Nl, Zvops[ZVOP_XD]);
 	    ft = 0;
 	}
 /*
@@ -894,7 +958,7 @@ vfs_read_error:
 	    goto vfs_read_error;
 /*
  * Read the afsnode, autonode, cnode, door_node, fifonode, fnnode, lnode,
- * inode, pcnode, rnode, snode, tmpnode, etc.
+ * inode, pcnode, rnode, snode, tmpnode, znode, etc.
  */
 	switch (Ntype) {
 	case N_SPEC:
@@ -1183,6 +1247,14 @@ vfs_read_error:
 	    break;
 #endif	/* defined(HASVXFS) */
 
+#if	defined(HAS_ZFS)
+	case N_ZFS:
+	    if (read_nzn(va, (KA_T)v->v_data, &zn))
+		return;
+	    zns = 1;
+	    break;
+#endif	/* defined(HAS_ZFS) */
+
 	case N_REGLR:
 	default:
 	    if (read_nin(va, (KA_T)v->v_data, &i))
@@ -1385,6 +1457,14 @@ vfs_read_error:
 		break;
 #endif	/* defined(HASVXFS) */
 
+#if	defined(HAS_ZFS)
+	    case N_ZFS:
+		if (read_nzn(va, (KA_T)v->v_data, &zn))
+		    return;
+		zns = 1;
+		break;
+#endif	/* defined(HAS_ZFS) */
+
 	    case N_REGLR:
 	    default:
 		if (read_nin(va, (KA_T)v->v_data, &i))
@@ -1815,6 +1895,25 @@ vfs_read_error:
 	    break;
 #endif	/* defined(HASVXFS) */
 
+#if	defined(HAS_ZFS)
+	case N_ZFS:
+	    if (zns) {
+		if (zn.z_zfsvfs
+		&&  !kread((KA_T)zn.z_zfsvfs, (char *)&zvfs, sizeof(zvfs))
+		&&  zvfs.z_vfs
+		&&  !kread((KA_T)zvfs.z_vfs, (char *)&zgvfs, sizeof(zgvfs))
+		) {
+		    dev = zgvfs.vfs_dev;
+		    devs = 1;
+		}
+	    }
+	    if ((v->v_type == VCHR) || (v->v_type == VBLK)) {
+		rdev = v->v_rdev;
+		rdevs = 1;
+	    }
+	    break;
+#endif	/* defined(HAS_ZFS) */
+
 	default:
 	    if (ins) {
 		dev = i.i_dev;
@@ -2036,6 +2135,15 @@ vfs_read_error:
 	    break;
 #endif	/* defined(HASVXFS) */
 
+#if	defined(HAS_ZFS)
+	case N_ZFS:
+	    if (zns) {
+		Lf->inode = (INODETYPE)zn.z_id;
+		Lf->inp_ty = 1;
+	    }
+	    break;
+#endif	/* defined(HAS_ZFS) */
+
 	}
 /*
  * Obtain the file size.
@@ -2209,12 +2317,46 @@ vfs_read_error:
 		break;
 #endif	/* defined(HASVXFS) */
 
+#if	defined(HAS_ZFS)
+	    case N_ZFS:
+		if (zns) {
+
+# if	HAS_ZFS!=1
+		/* HACK! */
+		/*
+		 * The following hack is made necessary by an inconsistency
+		 * between the definition of znode_t used by the kernel and
+		 * the one found in <sys/zfs_znode.h>.
+		 */
+		    zn.z_phys = (znode_phys_t *)zn.z_link_node.list_prev;
+		/* HACK! */
+# endif	/* HAS_ZFS!=1 */
+
+		    if (zn.z_phys
+		    &&  !kread((KA_T)zn.z_phys, (char *)&znp, sizeof(znp))
+		    ) {
+			zps = 1;
+			if (type == VREG || type == VDIR) {
+			    Lf->sz = (SZOFFTYPE)znp.zp_size;
+			    Lf->sz_def = 1;
+			} else if ((type == VCHR || type == VBLK) && !Fsize)
+			    Lf->off_def = 1;
+		    }
+		}
+		break;
+#endif	/* defined(HAS_ZFS) */
+
 	    }
 	}
 /*
  * Record link count.
  */
-	if (Fnlink) {
+
+#if	!defined(HASXOPT)
+	if (Fnlink)
+#endif	/* !defined(HASXOPT) */
+
+	{
 	    switch (Ntype) {
 
 #if	defined(HAS_AFS)
@@ -2337,6 +2479,15 @@ vfs_read_error:
 		break;
 #endif	/* defined(HASVXFS) */
 
+#if	defined(HAS_ZFS)
+	    case N_ZFS:
+		if (zps) {
+		    Lf->nlink = (long)MIN(znp.zp_links, UINT32_MAX);
+		    Lf->nlink_def = 1;
+		}
+		break;
+#endif	/* defined(HAS_ZFS) */
+
 	    }
 	    if (Nlink && Lf->nlink_def && (Lf->nlink < Nlink))
 		Lf->sf |= SELNLINK;
@@ -2609,11 +2760,7 @@ vfs_read_error:
 	    Lf->dev_def = devs;
 	    Lf->rdev = rdev;
 	    Lf->rdev_def = rdevs;
-	    if (type > 9999)
-		(void) snpf(Lf->type, sizeof(Lf->type), "*%03d", type % 1000);
-	    else
-		(void) snpf(Lf->type, sizeof(Lf->type), "%4d", type);
-	    (void) snpf(Namech, Namechl, "unknown type");
+	    (void) snpf(Lf->type, sizeof(Lf->type), "%04o", (type & 0xfff));
 	    ty = (char *)NULL;
 	}
 	if (ty)
@@ -3572,15 +3719,11 @@ read_npi(na, v, pids)
 	default:
 	    ty = (char *)NULL;
 	}
-	if (!ty) {
-	    if (pr.pr_type > 9999)
-		(void) snpf(Lf->type, sizeof(Lf->type), "*%03d", pr.pr_type);
-	    else
-		(void) snpf(Lf->type, sizeof(Lf->type), "%4d", pr.pr_type);
-	    (void) snpf(Namech, Namechl,
-		"unknown %s node type", HASPROCFS);
-	} else
+	if (ty)
 	    (void) snpf(Lf->type, sizeof(Lf->type), "%s", ty);
+	else
+	    (void) snpf(Lf->type, sizeof(Lf->type), "%04o",
+		(pr.pr_type & 0xfff));
 /*
  * Record the Solaris 2.6 /proc file system inode number.
  */
@@ -3819,6 +3962,31 @@ read_nvn(na, va, v)
 }
 
 
+#if	defined(HAS_ZFS)
+/*
+ * read_nzn() - read node's ZFS node
+ */
+
+static int
+read_nzn(na, nza, zn)
+	KA_T na;			/* containing node's address */
+	KA_T nza;			/* namenode address */
+	znode_t *zn;			/* znode receiver */
+{
+	char tbuf[32];
+
+	if (!nza || kread((KA_T)nza, (char *)zn, sizeof(znode_t))) {
+	    (void) snpf(Namech, Namechl, "node at %s: can't read znode: %s",
+		print_kptr(na, tbuf, sizeof(tbuf)),
+		print_kptr(nza, (char *)NULL, 0));
+	    enter_nm(Namech);
+	    return(1);
+	}
+	return(0);
+}
+#endif	/* defined(HAS_ZFS) */
+
+
 #if	solaris<100000
 /*
  * savesockmod() - save addresses from sockmod so_so structure
@@ -4025,6 +4193,13 @@ vop2ty(vp)
 	}
 # endif	/* defined(HASVXFS) */
 
+#if	defined(HAS_ZFS)
+	for (i = 0; i < ZVOP_NUM; i++) {
+	    if (Zvops[i] && Zvops[i] == (KA_T)vp->v_op)
+		return(N_ZFS);
+	}
+#endif	/* defined(HAS_ZFS) */
+
 	if (Tvops && Tvops == (KA_T)vp->v_op)
 	    return(N_TMP);
 	else if (Auvops && Auvops == (KA_T)vp->v_op)
@@ -4062,6 +4237,10 @@ vop2ty(vp)
 	    return(N_SAMFS);
 	else if (Sam2vops && Sam2vops == (KA_T)vp->v_op)
 	    return(N_SAMFS);
+	else if (Sam3vops && Sam3vops == (KA_T)vp->v_op)
+	    return(N_SAMFS);
+	else if (Sam4vops && Sam4vops == (KA_T)vp->v_op)
+	    return(N_SAMFS);
 	else if ((Sckvops  && Sckvops  == (KA_T)vp->v_op)
 	     ||  (Sncavops && Sncavops == (KA_T)vp->v_op)
 	     ||  (Stpivops && Stpivops == (KA_T)vp->v_op)
