@@ -32,7 +32,7 @@
 #ifndef lint
 static char copyright[] =
 "@(#) Copyright 1994 Purdue Research Foundation.\nAll rights reserved.\n";
-static char *rcsid = "$Id: dsock.c,v 1.26 2005/08/08 19:55:41 abe Exp $";
+static char *rcsid = "$Id: dsock.c,v 1.27 2007/04/24 16:23:15 abe Exp $";
 #endif
 
 
@@ -685,8 +685,8 @@ process_socket(sa, ty)
 		    if (tc.tcp_base
 		    &&  !kread((KA_T)tc.tcp_base, (char *)&tcb, sizeof(tcb)))
 			tcs = 1;
-		    tc.tcp_base = &tcb;		/* support for macroes */
-		    tcb.tcpb_tcp = &tc;		/* support for macroes */
+		    tc.tcp_base = &tcb;		/* support for macros */
+		    tcb.tcpb_tcp = &tc;		/* support for macros */
 		}
 # else	/* solaris<80000 */
 		    tcs = 1;
