@@ -32,7 +32,7 @@
 #ifndef lint
 static char copyright[] =
 "@(#) Copyright 1994 Purdue Research Foundation.\nAll rights reserved.\n";
-static char *rcsid = "$Id: dstore.c,v 1.19 2005/08/29 10:24:25 abe Exp $";
+static char *rcsid = "$Id: dstore.c,v 1.21 2007/04/24 16:23:15 abe Exp $";
 #endif
 
 
@@ -67,81 +67,89 @@ major_t CloneMaj;			/* clone major device number */
  */
 
 struct drive_Nl Drive_Nl[] = {
-	{ "arFid",	"afs_rootFid"		},
-	{ "avops",	"afs_vnodeops"		},
-	{ "Avops",	"Afs_vnodeops"		},
-	{ "avol",	"afs_volumes"		},
-	{ "auvops",	"auto_vnodeops"		},
-	{ "ctfsadir",	"ctfs_ops_adir"		},
-	{ "ctfsbund",	"ctfs_ops_bundle"	},
-	{ "ctfscdir",	"ctfs_ops_cdir"		},
-	{ "ctfsctl",	"ctfs_ops_ctl",		},
-	{ "ctfsevt",	"ctfs_ops_event",	},
-	{ "ctfslate",	"ctfs_ops_latest",	},
-	{ "ctfsroot",	"ctfs_ops_root",	},
-	{ "ctfsstat",	"ctfs_ops_stat",	},
-	{ "ctfssym",	"ctfs_ops_sym",		},
-	{ "ctfstdir",	"ctfs_ops_tdir",	},
-	{ "ctfstmpl",	"ctfs_ops_tmpl",	},
-	{ "cvops",	"cachefs_vnodeops"	},
-	{ "clmaj",	"clonemaj"		},
-	{ "fdops",	"fdvnodeops"		},
-	{ "fvops",	"fifo_vnodeops"		},
-	{ "hvops",	"hsfs_vnodeops"		},
-	{ "lvops",	"lo_vnodeops"		},
-	{ "mntops",	"mntvnodeops"		},
-	{ "mvops",	"mvfs_vnodeops"		},
+	{ "afsops",	"afs_ops"		 },
+	{ "arFid",	"afs_rootFid"		 },
+	{ "avops",	"afs_vnodeops"		 },
+	{ "Avops",	"Afs_vnodeops"		 },
+	{ "avol",	"afs_volumes"		 },
+	{ "auvops",	"auto_vnodeops"		 },
+	{ "ctfsadir",	"ctfs_ops_adir"		 },
+	{ "ctfsbund",	"ctfs_ops_bundle"	 },
+	{ "ctfscdir",	"ctfs_ops_cdir"		 },
+	{ "ctfsctl",	"ctfs_ops_ctl",		 },
+	{ "ctfsevt",	"ctfs_ops_event",	 },
+	{ "ctfslate",	"ctfs_ops_latest",	 },
+	{ "ctfsroot",	"ctfs_ops_root",	 },
+	{ "ctfsstat",	"ctfs_ops_stat",	 },
+	{ "ctfssym",	"ctfs_ops_sym",		 },
+	{ "ctfstdir",	"ctfs_ops_tdir",	 },
+	{ "ctfstmpl",	"ctfs_ops_tmpl",	 },
+	{ "cvops",	"cachefs_vnodeops"	 },
+	{ "clmaj",	"clonemaj"		 },
+	{ "fdops",	"fdvnodeops"		 },
+	{ "fvops",	"fifo_vnodeops"		 },
+	{ "hvops",	"hsfs_vnodeops"		 },
+	{ "lvops",	"lo_vnodeops"		 },
+	{ "mntops",	"mntvnodeops"		 },
+	{ "mvops",	"mvfs_vnodeops"		 },
 
 #if	solaris<90000
-	{ X_NCACHE,	"ncache"		},
-	{ X_NCSIZE,	"ncsize"		},
+	{ X_NCACHE,	"ncache"		 },
+	{ X_NCSIZE,	"ncsize"		 },
 #else	/* solaris>=90000 */
-	{ X_NCACHE,	"nc_hash"		},
-	{ X_NCSIZE,	"nc_hashsz"		},
-	{ "hshav",	"nc_hashavelen"		},
+	{ X_NCACHE,	"nc_hash"		 },
+	{ X_NCSIZE,	"nc_hashsz"		 },
+	{ "hshav",	"nc_hashavelen"		 },
 #endif	/* solaris<90000 */
 
 #if	defined(NCACHE_NEGVN)
-	{ NCACHE_NEGVN,	NCACHE_NEGVN		},
+	{ NCACHE_NEGVN,	NCACHE_NEGVN		 },
 #endif	/* defined(NCACHE_NEGVN) */
 
-	{ "nvops",	"nfs_vnodeops"		},
-	{ "n3vops",	"nfs3_vnodeops"		},
-	{ "n4vops",	"nfs4_vnodeops"		},
-	{ "nmvops",	"nm_vnodeops"		},
-        { "nproc",	"nproc"			},
-	{ "pdvops",	"pcfs_dvnodeops"	},
-	{ "pfvops",	"pcfs_fvnodeops"	},
-	{ "portvops",	"port_vnodeops"		},
-	{ "pract",	"practive"		},
-	{ "prvops",	"prvnodeops"		},
-	{ "sam1vops",	"samfs_vnodeops"	},
-	{ "sam2vops",	"samfs_client_vnodeops"	},
-	{ "sgvops",	"segvn_ops"		},
-	{ "sckvops",	"sock_vnodeops"		},
-	{ "spvops",	"spec_vnodeops"		},
-	{ "sncavops",	"socknca_vnodeops"	},
-	{ "stpivops",	"socktpi_vnodeops"	},
-	{ "tvops",	"tmp_vnodeops"		},
-	{ "uvops",	"ufs_vnodeops"		},
-	{ "vvfops",	"fdd_vnops"		},
-	{ "vvfcops",	"fdd_chain_vnops"	},
-	{ "vvfclops",	"vx_fcl_vnodeops_p"	},
-	{ "vvops",	"vx_vnodeops"		},
-	{ "vvops_p",	"vx_vnodeops_p"		},
+	{ "nvops",	"nfs_vnodeops"		 },
+	{ "n3vops",	"nfs3_vnodeops"		 },
+	{ "n4vops",	"nfs4_vnodeops"		 },
+	{ "nmvops",	"nm_vnodeops"		 },
+        { "nproc",	"nproc"			 },
+	{ "pdvops",	"pcfs_dvnodeops"	 },
+	{ "pfvops",	"pcfs_fvnodeops"	 },
+	{ "portvops",	"port_vnodeops"		 },
+	{ "pract",	"practive"		 },
+	{ "prvops",	"prvnodeops"		 },
+	{ "sam1vops",	"samfs_vnodeops"	 },
+	{ "sam2vops",	"samfs_client_vnodeops"	 },
+	{ "sam3vops",	"samfs_vnodeopsp"	 },
+	{ "sam4vops",	"samfs_client_vnodeopsp" },
+	{ "sgvops",	"segvn_ops"		 },
+	{ "sckvops",	"sock_vnodeops"		 },
+	{ "spvops",	"spec_vnodeops"		 },
+	{ "sncavops",	"socknca_vnodeops"	 },
+	{ "stpivops",	"socktpi_vnodeops"	 },
+	{ "tvops",	"tmp_vnodeops"		 },
+	{ "uvops",	"ufs_vnodeops"		 },
+	{ "vvfops",	"fdd_vnops"		 },
+	{ "vvfcops",	"fdd_chain_vnops"	 },
+	{ "vvfclops",	"vx_fcl_vnodeops_p"	 },
+	{ "vvops",	"vx_vnodeops"		 },
+	{ "vvops_p",	"vx_vnodeops_p"		 },
 
 #if	solaris>=20500
-	{ "devops",	"dv_vnodeops"		},
-	{ "dvops",	"door_vnodeops"		},
-	{ "kbase",	"_kernelbase"		},
+	{ "devops",	"dv_vnodeops"		 },
+	{ "dvops",	"door_vnodeops"		 },
+	{ "kbase",	"_kernelbase"		 },
 #endif	/* solaris>=20500 */
 
 #if	solaris>=20501
-	{ "kasp",	"kas"			},
+	{ "kasp",	"kas"			 },
 #endif	/* solaris>=20501 */
 
-	{ "",		""			},
-	{ NULL,		NULL			}
+	{ "zfsdops",	"zfs_dvnodeops"		 },
+	{ "zfseops",	"zfs_evnodeops"		 },
+	{ "zfsfops",	"zfs_fvnodeops"		 },
+	{ "zfssymops",	"zfs_symvnodeops"	 },
+	{ "zfsxdops",	"zfs_xdvnodeops"	 },
+	{ "",		""			 },
+	{ NULL,		NULL			 }
 };
 
 char **Fsinfo = NULL;			/* file system information */
