@@ -31,7 +31,7 @@
 
 
 /*
- * $Id: machine.h,v 1.40 2005/08/29 10:22:41 abe Exp $
+ * $Id: machine.h,v 1.42 2007/04/24 16:23:15 abe Exp $
  */
 
 
@@ -76,6 +76,7 @@ typedef struct aio_req { int dummy; } aio_req_t;
 
 # if	solaris>=80000
 #include <sys/wait.h>
+#include <sys/types32.h>
 #define	_KERNEL	1
 #include <netinet/in.h>
 #undef	_KERNEL
@@ -194,10 +195,22 @@ typedef struct aio_req { int dummy; } aio_req_t;
  * FSV_DEFAULT defines the default set of file structure values to list.
  * It defaults to zero (0), but may be made up of a combination of the
  * FSV_* symbols from lsof.h.
+ *
+ * If any file structure value is unavailable, its use may be suppressed
+ * with any of the following definitions:
+ *
+ *   HASNOFSADDR  -- has no file structure address
+ *   HASNOFSFLAGS -- has no file structure flags
+ *   HASNOFSCOUNT -- has no file structure count
+ *   HASNOFSNADDR -- has no file structure node address
  */
 
 #define	HASFSTRUCT	1
 /* #define	FSV_DEFAULT	FSV_? | FSV_? | FSV_? */
+/* #define	HASNOFSADDR	1	has no file structure address */
+/* #define	HASNOFSFLAGS	1	has no file structure flags */
+/* #define	HASNOFSCOUNT	1	has no file structure count */
+/* #define	HASNOFSNADDR	1	has no file structure node address */
 
 
 /*
@@ -580,8 +593,10 @@ typedef struct aio_req { int dummy; } aio_req_t;
  * option's default binary value -- 0 or 1.
  */
 
-/* #define	HASXOPT		"help text for X option" */
-/* #define	HASXOPT_VALUE	1 */
+# if	solaris>=10000 && defined(HAS_V_PATH)
+#define	HASXOPT		"report deleted paths"
+#define	HASXOPT_VALUE	0
+# endif	/* solaris>=10000 && defined(HAS_V_PATH) */
 
 
 /*
