@@ -32,7 +32,7 @@
 #ifndef lint
 static char copyright[] =
 "@(#) Copyright 1996 Purdue Research Foundation.\nAll rights reserved.\n";
-static char *rcsid = "$Id: dsock.c,v 1.15 2004/03/10 23:52:32 abe Exp $";
+static char *rcsid = "$Id: dsock.c,v 1.16 2005/08/13 16:21:41 abe Exp $";
 #endif
 
 
@@ -179,7 +179,8 @@ print_tcptpi(nl)
 	    int opt;
 
 	    if ((opt = Lf->lts.opt)
-	    ||  Lf->lts.qlens || Lf->lts.qlims || Lf->lts.rbszs || Lf->lts.sbsz
+	    ||  Lf->lts.pqlens || Lf->lts.qlens || Lf->lts.qlims
+	    ||  Lf->lts.rbszs  || Lf->lts.sbsz
 	    ) {
 		char sep = ' ';
 
@@ -289,12 +290,16 @@ print_tcptpi(nl)
 		}
 # endif	/* defined(SO_PARALLELSVR) */
 
+		if (Lf->lts.pqlens) {
+		    (void) printf("%cPQLEN=%u", sep, Lf->lts.pqlen);
+		    sep = ',';
+		}
 		if (Lf->lts.qlens) {
-		    (void) printf("%cQLEN=%d", sep, Lf->lts.qlen);
+		    (void) printf("%cQLEN=%u", sep, Lf->lts.qlen);
 		    sep = ',';
 		}
 		if (Lf->lts.qlims) {
-		    (void) printf("%cQLIM=%lu", sep, Lf->lts.qlim);
+		    (void) printf("%cQLIM=%u", sep, Lf->lts.qlim);
 		    sep = ',';
 		}
 		if (Lf->lts.rbszs) {
@@ -871,12 +876,13 @@ process_socket(pr, q)
 #if	defined(HASSOOPT)
 	    Lf->lts.opt = (unsigned int)inp.inp_protoopt;
 	    Lf->lts.ltm = (unsigned int)inp.inp_linger;
+	    Lf->lts.pqlen = (unsigned int)t.t_q0len;
 	    Lf->lts.qlen = (unsigned int)t.t_qlen;
 	    Lf->lts.qlim = (unsigned int)t.t_qlimit;
 	    Lf->lts.rbsz = (unsigned long)inp.inp_rbufsize;
 	    Lf->lts.sbsz = (unsigned long)inp.inp_sbufsize;
-	    Lf->lts.qlens = Lf->lts.qlims = Lf->lts.rbszs
-			  = Lf->lts.sbszs = (unsigned char)1;
+	    Lf->lts.pqlens = Lf->lts.qlens = Lf->lts.qlims = Lf->lts.rbszs
+			   = Lf->lts.sbszs = (unsigned char)1;
 #endif	/* defined(HASSOOPT) */
 
 #if	defined(HASSOSTATE)
