@@ -37,7 +37,7 @@
 # if	!defined(lint)
 static char copyright[] =
 "@(#) Copyright 1997 Purdue Research Foundation.\nAll rights reserved.\n";
-static char *rcsid = "$Id: ptti.c,v 1.3 2004/03/11 10:54:36 abe Exp $";
+static char *rcsid = "$Id: ptti.c,v 1.5 2005/08/08 19:41:06 abe Exp $";
 # endif	/* !defined(lint) */
 
 #define	TCPSTATES			/* activate tcpstates[] */
@@ -107,7 +107,8 @@ print_tcptpi(nl)
 	    int opt;
 
 	    if ((opt = Lf->lts.opt)
-	    ||  Lf->lts.qlens || Lf->lts.qlims || Lf->lts.rbszs || Lf->lts.sbsz
+	    ||  Lf->lts.pqlens || Lf->lts.qlens || Lf->lts.qlims
+	    ||  Lf->lts.rbszs  || Lf->lts.sbsz
 	    ) {
 		char sep = ' ';
 
@@ -331,12 +332,16 @@ print_tcptpi(nl)
 		}
 # endif	/* defined(SO_ORDREL) */
 
+		if (Lf->lts.pqlens) {
+		    (void) printf("%cPQLEN=%u", sep, Lf->lts.pqlen);
+		    sep = ',';
+		}
 		if (Lf->lts.qlens) {
-		    (void) printf("%cQLEN=%d", sep, Lf->lts.qlen);
+		    (void) printf("%cQLEN=%u", sep, Lf->lts.qlen);
 		    sep = ',';
 		}
 		if (Lf->lts.qlims) {
-		    (void) printf("%cQLIM=%lu", sep, Lf->lts.qlim);
+		    (void) printf("%cQLIM=%u", sep, Lf->lts.qlim);
 		    sep = ',';
 		}
 		if (Lf->lts.rbszs) {
@@ -447,7 +452,7 @@ print_tcptpi(nl)
 
 #if	defined(HASSOSTATE)
 	if (Ftcptpi & TCPTPI_FLAGS) {
-	    int ss;
+	    unsigned int ss;
 
 	    if ((ss = Lf->lts.ss)) {
 		char sep = ' ';
@@ -476,21 +481,40 @@ print_tcptpi(nl)
 		}
 # endif	/* defined(SS_BOUND) */
 
-# if	defined(SS_CANTRCVMORE)
+# if	defined(HASSBSTATE)
+#  if	defined(SBS_CANTRCVMORE)
+		if (Lf->lts.sbs_rcv & SBS_CANTRCVMORE) {
+		    (void) printf("%cCANTRCVMORE", sep);
+		    Lf->lts.sbs_rcv &= ~SBS_CANTRCVMORE;
+		    sep = ',';
+		}
+#  endif	/* defined(SBS_CANTRCVMORE) */
+
+#  if	defined(SBS_CANTSENDMORE)
+		if (Lf->lts.sbs_snd & SBS_CANTSENDMORE) {
+		    (void) printf("%cCANTSENDMORE", sep);
+		    Lf->lts.sbs_snd &= ~SBS_CANTSENDMORE;
+		    sep = ',';
+		}
+#  endif	/* defined(SS_CANTSENDMORE) */
+# else	/* !defined(HASSBSTATE) */
+
+#  if	defined(SS_CANTRCVMORE)
 		if (ss & SS_CANTRCVMORE) {
 		    (void) printf("%cCANTRCVMORE", sep);
 		    ss &= ~SS_CANTRCVMORE;
 		    sep = ',';
 		}
-# endif	/* defined(SS_CANTRCVMORE) */
+#  endif	/* defined(SS_CANTRCVMORE) */
 
-# if	defined(SS_CANTSENDMORE)
+#  if	defined(SS_CANTSENDMORE)
 		if (ss & SS_CANTSENDMORE) {
 		    (void) printf("%cCANTSENDMORE", sep);
 		    ss &= ~SS_CANTSENDMORE;
 		    sep = ',';
 		}
-# endif	/* defined(SS_CANTSENDMORE) */
+#  endif	/* defined(SS_CANTSENDMORE) */
+# endif	/* defined(HASSBSTATE) */
 
 # if	defined(SS_COMP)
 		if (ss & SS_COMP) {
@@ -644,13 +668,24 @@ print_tcptpi(nl)
 		}
 # endif	/* defined(SS_QUEUE) */
 
-# if	defined(SS_RCVATMARK)
+# if	defined(HASSBSTATE)
+#  if	defined(SBS_RCVATMARK)
+		if (Lf->lts.sbs_rcv & SBS_RCVATMARK) {
+		    (void) printf("%cRCVATMARK", sep);
+		    Lf->lts.sbs_rcv &= ~SBS_RCVATMARK;
+		    sep = ',';
+		}
+#  endif	/* defined(SBS_RCVATMARK) */
+
+# else	/* !defined(HASSBSTATE) */
+#  if	defined(SS_RCVATMARK)
 		if (ss & SS_RCVATMARK) {
 		    (void) printf("%cRCVATMARK", sep);
 		    ss &= ~SS_RCVATMARK;
 		    sep = ',';
 		}
-# endif	/* defined(SS_RCVATMARK) */
+#  endif	/* defined(SS_RCVATMARK) */
+# endif	/* defined(HASSBSTATE) */
 
 # if	defined(SS_READWAIT)
 		if (ss & SS_READWAIT) {
