@@ -32,7 +32,7 @@
 #ifndef lint
 static char copyright[] =
 "@(#) Copyright 1994 Purdue Research Foundation.\nAll rights reserved.\n";
-static char *rcsid = "$Id: print.c,v 1.46 2005/08/08 19:44:08 abe Exp $";
+static char *rcsid = "$Id: print.c,v 1.48 2006/09/17 17:49:34 abe Exp $";
 #endif
 
 
@@ -651,9 +651,14 @@ print_file()
 
 #if	defined(HASZONES)
 	    if (Fzone)
-	        (void) printf(" %-*s", ZoneColW, ZONETTL);
+		(void) printf(" %-*s", ZoneColW, ZONETTL);
 #endif	/* defined(HASZONES) */
 
+#if	defined(HASSELINUX)
+	    if (Fcntx)
+		(void) printf(" %-*s", CntxColW, CNTXTTL);
+#endif /* defined(HASSELINUX) */
+
 #if	defined(HASPPID)
 	    if (Fppid)
 	 	(void) printf(" %*s", PpidColW, PPIDTTL);
@@ -668,14 +673,27 @@ print_file()
 
 #if	defined(HASFSTRUCT)
 	    if (Fsv) {
+
+# if	!defined(HASNOFSADDR)
 		if (Fsv & FSV_FA)
 		    (void) printf(" %*s", FsColW, FSTTL);
+# endif	/* !defined(HASNOFSADDR) */
+
+# if	!defined(HASNOFSCOUNT)
 		if (Fsv & FSV_CT)
 		    (void) printf(" %*s", FcColW, FCTTL);
+# endif	/* !defined(HASNOFSCOUNT) */
+
+# if	!defined(HASNOFSFLAGS)
 		if (Fsv & FSV_FG)
 		    (void) printf(" %*s", FgColW, FGTTL);
+# endif	/* !defined(HASNOFSFLAGS) */
+
+# if	!defined(HASNOFSNADDR)
 		if (Fsv & FSV_NI)
 		    (void) printf(" %*s", NiColW, NiTtl);
+# endif	/* !defined(HASNOFSNADDR) */
+
 	    }
 #endif	/* defined(HASFSTRUCT) */
 
@@ -728,6 +746,21 @@ print_file()
 	}
 #endif	/* defined(HASZONES) */
 
+#if	defined(HASSELINUX)
+/*
+ * Size or print the context.
+ */
+	if (Fcntx) {
+	    if (!PrPass) {
+		if (Lp->cntx) {
+		    if ((len = strlen(Lp->cntx)) > CntxColW)
+			CntxColW = len;
+		}
+	    } else
+		(void) printf(" %-*s", CntxColW, Lp->cntx ? Lp->cntx : "");
+	}
+#endif	/* defined(HASSELINUX) */
+
 #if	defined(HASPPID)
 	if (Fppid) {
 
@@ -798,6 +831,8 @@ print_file()
  */
 
 	if (Fsv) {
+
+# if	!defined(HASNOFSADDR)
 	    if (Fsv & FSV_FA) {
 		cp =  (Lf->fsv & FSV_FA) ? print_kptr(Lf->fsa, buf, sizeof(buf))
 					 : "";
@@ -808,6 +843,9 @@ print_file()
 		    (void) printf(" %*.*s", FsColW, FsColW, cp);
 		    
 	    }
+# endif	/* !defined(HASNOFSADDR) */
+
+# if	!defined(HASNOFSCOUNT)
 	    if (Fsv & FSV_CT) {
 		if (Lf->fsv & FSV_CT) {
 		    (void) snpf(buf, sizeof(buf), "%ld", Lf->fct);
@@ -820,6 +858,9 @@ print_file()
 		} else
 		    (void) printf(" %*.*s", FcColW, FcColW, cp);
 	    }
+# endif	/* !defined(HASNOFSCOUNT) */
+
+# if	!defined(HASNOFSFLAGS)
 	    if (Fsv & FSV_FG) {
 		if ((Lf->fsv & FSV_FG) && (FsvFlagX || Lf->ffg || Lf->pof))
 		    cp = print_fflags(Lf->ffg, Lf->pof);
@@ -831,6 +872,9 @@ print_file()
 		} else
 		    (void) printf(" %*.*s", FgColW, FgColW, cp);
 	    }
+# endif	/* !defined(HASNOFSFLAGS) */
+
+# if	!defined(HASNOFSNADDR)
 	    if (Fsv & FSV_NI) {
 		cp = (Lf->fsv & FSV_NI) ? print_kptr(Lf->fna, buf, sizeof(buf))
 					: "";
@@ -840,6 +884,8 @@ print_file()
 		} else
 		    (void) printf(" %*.*s", NiColW, NiColW, cp);
 	    }
+# endif	/* !defined(HASNOFSNADDR) */
+
 	}
 #endif	/* defined(HASFSTRUCT) */
 
@@ -1186,12 +1232,29 @@ print_init()
 	UserColW = strlen(USERTTL);
 
 #if	defined(HASFSTRUCT)
-	FcColW = strlen(FCTTL);
-	FgColW = strlen(FGTTL);
+
+# if	!defined(HASNOFSADDR)
 	FsColW = strlen(FSTTL);
+# endif	/* !defined(HASNOFSADDR) */
+
+# if	!defined(HASNOFSCOUNT)
+	FcColW = strlen(FCTTL);
+# endif	/* !defined(HASNOFSCOUNT) */
+
+# if	!defined(HASNOFSFLAGS)
+	FgColW = strlen(FGTTL);
+# endif	/* !defined(HASNOFSFLAGS) */
+
+# if	!defined(HASNOFSNADDR)
 	NiColW = strlen(NiTtl);
+# endif	/* !defined(HASNOFSNADDR) */
 #endif	/* defined(HASFSTRUCT) */
 
+#if	defined(HASSELINUX)
+	if (Fcntx)
+	    CntxColW = strlen(CNTXTTL);
+#endif	/* defined(HASSELINUX) */
+
 #if	defined(HASZONES)
 	if (Fzone)
 	    ZoneColW = strlen(ZONETTL);
@@ -1867,7 +1930,7 @@ printname(nl)
 
 #if	defined(HASNCACHE)
 	char buf[MAXPATHLEN];
-	char *cp, *cp1;
+	char *cp;
 	int fp;
 #endif	/* defined(HASNCACHE) */
 
@@ -1904,7 +1967,6 @@ printname(nl)
 	    ps++;
 	    goto print_nma;
 	}
-
 	if (Lf->is_com) {
 
 	/*
@@ -1916,8 +1978,10 @@ printname(nl)
 	}
 
 #if	defined(HASPRIVNMCACHE)
-	if (HASPRIVNMCACHE(Lf))
+	if (HASPRIVNMCACHE(Lf)) {
+	    ps++;
 	    goto print_nma;
+	}
 #endif	/* defined(HASPRIVNMCACHE) */
 
 	if (Lf->lmi_srch) {
@@ -1973,6 +2037,8 @@ printname(nl)
 		    NcacheReload = 0;
 		}
 		if ((cp = ncache_lookup(buf, sizeof(buf), &fp))) {
+		    char *cp1; 
+
 		    if (*cp == '\0')
 			goto print_nma;
 		    if (fp && Lf->fsdir) {
@@ -2228,7 +2294,7 @@ printuid(uid, ty)
 			}
 		    }
 		    sbs = sb;
-	        }
+		}
 		CkPasswd = 0;
 	    }
 	/*
